@@ -1,15 +1,4 @@
-use ahash::AHashMap;
-use std::io;
-use std::iter;
-use std::ops;
-
-use crate::config::{Config, Delimiter};
-use crate::select::{SelectColumns, Selection};
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Fill empty fields in selected columns of a CSV.
 
 This command fills empty fields in the selected column
@@ -40,6 +29,8 @@ CSV is not sorted by the `--groupby` columns, rows may be
 re-ordered during output due to the buffering of rows
 collected before the first valid value.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_fill.rs.
+
 Usage:
     qsv fill [options] [--] <selection> [<input>]
     qsv fill --help
@@ -58,24 +49,34 @@ Common options:
                            sliced, etc.)
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::{io, iter, ops};
+
+use ahash::AHashMap;
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    select::{SelectColumns, Selection},
+    util, CliResult,
+};
 
 type ByteString = Vec<u8>;
-
 type BoxedWriter = csv::Writer<Box<dyn io::Write + 'static>>;
 type BoxedReader = csv::Reader<Box<dyn io::Read + 'static>>;
 
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
-    arg_selection: SelectColumns,
-    flag_output: Option<String>,
+    arg_input:       Option<String>,
+    arg_selection:   SelectColumns,
+    flag_output:     Option<String>,
     flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
-    flag_groupby: Option<SelectColumns>,
-    flag_first: bool,
-    flag_backfill: bool,
-    flag_default: Option<String>,
+    flag_delimiter:  Option<Delimiter>,
+    flag_groupby:    Option<SelectColumns>,
+    flag_first:      bool,
+    flag_backfill:   bool,
+    flag_default:    Option<String>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -84,6 +85,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
+        .checkutf8(false)
         .select(args.arg_selection);
 
     let wconfig = Config::new(&args.flag_output);
@@ -114,6 +116,7 @@ struct ByteRecord(Vec<ByteString>);
 
 impl<'a> From<&'a csv::ByteRecord> for ByteRecord {
     fn from(record: &'a csv::ByteRecord) -> Self {
+        #[allow(clippy::redundant_closure_for_method_calls)]
         ByteRecord(record.iter().map(|f| f.to_vec()).collect())
     }
 }
@@ -159,7 +162,7 @@ impl GroupKeyConstructor for GroupKeySelection {
 
 #[derive(Debug)]
 struct GroupValues {
-    map: AHashMap<usize, ByteString>,
+    map:     AHashMap<usize, ByteString>,
     default: Option<ByteString>,
 }
 
@@ -214,12 +217,12 @@ impl GroupMemorizer for GroupValues {
 }
 
 struct Filler {
-    grouper: Grouper,
-    groupby: GroupKeySelection,
-    select: Selection,
-    buffer: GroupBuffer,
-    first: bool,
-    backfill: bool,
+    grouper:       Grouper,
+    groupby:       GroupKeySelection,
+    select:        Selection,
+    buffer:        GroupBuffer,
+    first:         bool,
+    backfill:      bool,
     default_value: Option<ByteString>,
 }
 
@@ -303,11 +306,11 @@ impl Filler {
 }
 
 struct MapSelected<I, F> {
-    selection: Vec<usize>,
+    selection:       Vec<usize>,
     selection_index: usize,
-    index: usize,
-    iterator: I,
-    predicate: F,
+    index:           usize,
+    iterator:        I,
+    predicate:       F,
 }
 
 impl<I: iter::Iterator, F> iter::Iterator for MapSelected<I, F>
@@ -317,10 +320,7 @@ where
     type Item = I::Item;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let item = match self.iterator.next() {
-            Some(item) => item,
-            None => return None,
-        };
+        let Some(item) = self.iterator.next() else { return None };
         let result = match self.selection_index {
             ref mut sidx if (self.selection.get(*sidx) == Some(&self.index)) => {
                 *sidx += 1;
