@@ -1,5 +1,7 @@
 #[cfg(all(feature = "apply", not(feature = "lite")))]
 pub mod apply;
+#[cfg(feature = "datapusher_plus")]
+pub mod applydp;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod behead;
 #[cfg(any(feature = "full", feature = "lite"))]
@@ -12,9 +14,12 @@ pub mod excel;
 pub mod exclude;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod explode;
+#[cfg(any(feature = "full", feature = "lite"))]
 pub mod extsort;
 #[cfg(all(feature = "fetch", not(feature = "lite")))]
 pub mod fetch;
+#[cfg(all(feature = "fetch", not(feature = "lite")))]
+pub mod fetchpost;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod fill;
 #[cfg(any(feature = "full", feature = "lite"))]
@@ -35,8 +40,8 @@ pub mod input;
 pub mod join;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod jsonl;
-#[cfg(all(feature = "lua", not(feature = "lite")))]
-pub mod lua;
+#[cfg(feature = "luau")]
+pub mod luau;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod partition;
 pub mod pseudo;
@@ -46,6 +51,7 @@ pub mod rename;
 pub mod replace;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod reverse;
+pub mod safenames;
 pub mod sample;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod schema;
@@ -55,11 +61,16 @@ pub mod select;
 pub mod slice;
 pub mod sniff;
 pub mod sort;
+pub mod sortcheck;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod split;
 pub mod stats;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod table;
+#[cfg(all(feature = "to", not(feature = "lite")))]
+pub mod to;
+#[cfg(any(feature = "full", feature = "lite"))]
+pub mod tojsonl;
 #[cfg(any(feature = "full", feature = "lite"))]
 pub mod transpose;
 pub mod validate;
