@@ -1,24 +1,11 @@
-use ahash::AHashMap;
-use std::collections::hash_map::Entry;
-use std::collections::HashSet;
-use std::fs;
-use std::io;
-use std::path::Path;
-
-use regex::Regex;
-
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util::{self, FilenameTemplate};
-use crate::CliResult;
-use serde::Deserialize;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Partitions the given CSV data into chunks based on the value of a column
 
 The files are written to the output directory with filenames based on the
 values in the partition column and the `--filename` flag.
 
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_partition.rs.
+
 Usage:
     qsv partition [options] <column> <outdir> [<input>]
     qsv partition --help
@@ -41,18 +28,35 @@ Common options:
                            appear in all chunks as the header row.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
-";
+"#;
+
+use std::{
+    collections::{hash_map::Entry, HashSet},
+    fs, io,
+    path::Path,
+};
+
+use ahash::AHashMap;
+use regex::Regex;
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util::{self, FilenameTemplate},
+    CliResult,
+};
 
 #[derive(Clone, Deserialize)]
 struct Args {
-    arg_column: SelectColumns,
-    arg_input: Option<String>,
-    arg_outdir: String,
-    flag_filename: FilenameTemplate,
+    arg_column:         SelectColumns,
+    arg_input:          Option<String>,
+    arg_outdir:         String,
+    flag_filename:      FilenameTemplate,
     flag_prefix_length: Option<usize>,
-    flag_drop: bool,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+    flag_drop:          bool,
+    flag_no_headers:    bool,
+    flag_delimiter:     Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -72,10 +76,12 @@ impl Args {
         Config::new(&self.arg_input)
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
+            .checkutf8(false)
             .select(self.arg_column.clone())
     }
 
     /// Get the column to use as a key.
+    #[allow(clippy::unused_self)]
     fn key_column(&self, rconfig: &Config, headers: &csv::ByteRecord) -> CliResult<usize> {
         let select_cols = rconfig.selection(headers)?;
         if select_cols.len() == 1 {
@@ -142,9 +148,9 @@ type BoxedWriter = csv::Writer<Box<dyn io::Write + 'static>>;
 
 /// Generates unique filenames based on CSV values.
 struct WriterGenerator {
-    template: FilenameTemplate,
-    counter: usize,
-    used: HashSet<String>,
+    template:      FilenameTemplate,
+    counter:       usize,
+    used:          HashSet<String>,
     non_word_char: Regex,
 }
 
@@ -173,7 +179,7 @@ impl WriterGenerator {
     fn unique_value(&mut self, key: &[u8]) -> String {
         // Sanitize our key.
         let utf8 = String::from_utf8_lossy(key);
-        let safe = self.non_word_char.replace_all(&*utf8, "").into_owned();
+        let safe = self.non_word_char.replace_all(&utf8, "").into_owned();
         let base = if safe.is_empty() {
             "empty".to_owned()
         } else {
@@ -181,10 +187,7 @@ impl WriterGenerator {
         };
 
         // Now check for collisions.
-        if !self.used.contains(&base) {
-            self.used.insert(base.clone());
-            base
-        } else {
+        if self.used.contains(&base) {
             loop {
                 let candidate = format!("{}_{}", &base, self.counter);
                 self.counter = self.counter.checked_add(1).unwrap_or_else(|| {
@@ -198,6 +201,9 @@ impl WriterGenerator {
                     return candidate;
                 }
             }
+        } else {
+            self.used.insert(base.clone());
+            base
         }
     }
 }
