@@ -1,9 +1,3 @@
-use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
-use crate::CliResult;
-use serde::Deserialize;
-
 static USAGE: &str = r#"
 Select columns from CSV data efficiently.
 
@@ -51,13 +45,21 @@ Common options:
                            Must be a single character. (default: ,)
 "#;
 
+use serde::Deserialize;
+
+use crate::{
+    config::{Config, Delimiter},
+    select::SelectColumns,
+    util, CliResult,
+};
+
 #[derive(Deserialize)]
 struct Args {
-    arg_input: Option<String>,
-    arg_selection: SelectColumns,
-    flag_output: Option<String>,
+    arg_input:       Option<String>,
+    arg_selection:   SelectColumns,
+    flag_output:     Option<String>,
     flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+    flag_delimiter:  Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -66,6 +68,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
+        .checkutf8(false)
         .select(args.arg_selection);
 
     let mut rdr = rconfig.reader()?;
