@@ -1,27 +1,37 @@
 static USAGE: &str = r#"
 Computes summary statistics on CSV data.
 
-Summary statistics includes sum, min/max, min/max length, mean, stddev, variance,
+Summary statistics includes sum, min/max/range, min/max length, mean, stddev, variance,
 nullcount, quartiles, interquartile range (IQR), lower/upper fences, skewness, median, 
-mode/s, & cardinality. Note that some statistics are expensive to compute and requires
-loading the entire file into memory, so they must be enabled explicitly. 
+cardinality, mode/s & antimode/s. Note that some statistics are expensive to compute and
+requires loading the entire file into memory, so they must be enabled explicitly. 
 
 By default, the following statistics are reported for *every* column in the CSV data:
-sum, min/max values, min/max length, mean, stddev, variance & nullcount. The default
+sum, min/max/range values, min/max length, mean, stddev, variance & nullcount. The default
 set of statistics corresponds to statistics that can be computed efficiently on a stream
 of data (i.e., constant memory) and can work with arbitrarily large CSV files.
 
-Note that standard deviation for dates is returned in days, not in seconds.
-
 The following additional statistics require loading the entire file into memory:
-mode (multimodal aware), cardinality, median, quartiles and its related measures 
-(IQR, lower/upper fences and skewness).
+cardinality, mode/antimode, median, quartiles and its related measures 
+(Interquartile Range (IQR), lower/upper fences & skewness).
+
+Antimode is the least frequently occurring non-zero score and is the opposite of mode.
+It return "*ALL" if all the values are unique, and only returns a preview of the first
+10 antimodes, for readability purposes.
+
+If you need all the values of a column, run the `frequency` command with --limit set to zero.
+The tail of the resulting frequency table for each column will have all its antimode values.
 
 Each column's data type is also inferred (NULL, Integer, String, Float, Date & DateTime). 
 Note that the Date and DateTime data types are only inferred with the --infer-dates option 
 as its an expensive operation. The date formats recognized can be found at 
 https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats.
 
+Summary statistics for dates are also computed when --infer-dates is enabled, with DateTime
+results in rfc3339 format and Date results in "yyyy-mm-dd" format. Date range & stddev are 
+returned in days, not seconds. Date variance is currently not computed as the current
+streaming variance algorithm is not well suited to unix epoch timestamp values.
+
 Unlike the sniff command, stats' data type inferences are GUARANTEED, as the entire file
 is scanned, and not just sampled.
 
@@ -51,6 +61,8 @@ stats options:
                               This requires loading all CSV data in memory.
     --round <decimal_places>  Round statistics to <decimal_places>. Rounding is done following
                               Midpoint Nearest Even (aka "Bankers Rounding") rule.
+                              Date range & stddev are always at least 5 decimal places as
+                              they are reported in days, and 5 places gives us millisecond precision.
                               [default: 4]
     --nulls                   Include NULLs in the population size for computing
                               mean and standard deviation.
@@ -93,7 +105,6 @@ use std::{
     sync::atomic::{AtomicBool, Ordering},
 };
 
-use chrono::prelude::*;
 use itertools::Itertools;
 use once_cell::sync::OnceCell;
 use qsv_dateparser::parse_with_preference;
@@ -321,14 +332,15 @@ impl Args {
             return csv::StringRecord::from(vec!["field", "type"]);
         }
 
-        // with --everything, we have 22 columns at most
-        let mut fields = Vec::with_capacity(22);
+        // with --everything, we have 28 columns at most
+        let mut fields = Vec::with_capacity(28);
         fields.extend_from_slice(&[
             "field",
             "type",
             "sum",
             "min",
             "max",
+            "range",
             "min_length",
             "max_length",
             "mean",
@@ -353,12 +365,17 @@ impl Args {
                 "skewness",
             ]);
         }
-        if self.flag_mode || all {
-            fields.push("mode");
-        }
         if self.flag_cardinality || all {
             fields.push("cardinality");
         }
+        if self.flag_mode || all {
+            fields.push("mode");
+            fields.push("mode_count");
+            fields.push("mode_occurrences");
+            fields.push("antimode");
+            fields.push("antimode_count");
+            fields.push("antimode_occurrences");
+        }
         csv::StringRecord::from(fields)
     }
 }
@@ -450,7 +467,13 @@ pub struct Stats {
 fn round_num(dec_f64: f64, places: u32) -> String {
     use rust_decimal::prelude::*;
 
-    let dec_num = Decimal::try_from(dec_f64).unwrap_or_default();
+    // use from_f64_retain, so we have all the excess bits before rounding with
+    // round_dp_with_strategy as from_f64 will prematurely round when it drops the excess bits
+    let Some(dec_num) = Decimal::from_f64_retain(dec_f64) else {
+        let msg = format!(r#"Failed to convert to decimal "{dec_f64}""#);
+        log::error!("{msg}");
+        return msg;
+    };
 
     // round using Midpoint Nearest Even Rounding Strategy AKA "Bankers Rounding."
     // https://docs.rs/rust_decimal/latest/rust_decimal/enum.RoundingStrategy.html#variant.MidpointNearestEven
@@ -459,12 +482,19 @@ fn round_num(dec_f64: f64, places: u32) -> String {
         .to_string()
 }
 
-fn unixtime_to_isodate(unixtime: f64) -> String {
-    chrono::DateTime::<Utc>::from_utc(
-        chrono::NaiveDateTime::from_timestamp_opt(unixtime as i64, 0).unwrap_or_default(),
-        chrono::offset::Utc,
+fn timestamp_ms_to_rfc3339(timestamp: i64, typ: FieldType) -> String {
+    use chrono::prelude::*;
+
+    let date_val = DateTime::<Utc>::from_utc(
+        NaiveDateTime::from_timestamp_millis(timestamp).unwrap_or_default(),
+        Utc,
     )
-    .to_string()
+    .to_rfc3339();
+
+    if typ == TDate {
+        return date_val[..10].to_string();
+    }
+    date_val
 }
 
 impl Stats {
@@ -564,7 +594,7 @@ impl Stats {
                     DMY_PREFERENCE.load(Ordering::Relaxed),
                 ) {
                     #[allow(clippy::cast_precision_loss)]
-                    let n = parsed_date.timestamp() as f64;
+                    let n = parsed_date.timestamp_millis() as f64;
                     if let Some(v) = self.median.as_mut() {
                         v.add(n);
                     }
@@ -590,8 +620,8 @@ impl Stats {
 
         let typ = self.typ;
         // prealloc memory for performance
-        // we have 22 columns at most with --everything
-        let mut pieces = Vec::with_capacity(22);
+        // we have 28 columns at most with --everything
+        let mut pieces = Vec::with_capacity(28);
         let empty = String::new;
 
         // type
@@ -600,15 +630,8 @@ impl Stats {
         // sum
         if let Some(sum) = self.sum.as_ref().and_then(|sum| sum.show(typ)) {
             if typ == FieldType::TFloat {
-                let sum_decimal = rust_decimal::Decimal::from_str(&sum).unwrap();
-                pieces.push(
-                    sum_decimal
-                        .round_dp_with_strategy(
-                            round_places,
-                            rust_decimal::RoundingStrategy::MidpointNearestEven,
-                        )
-                        .to_string(),
-                );
+                let f64_val = sum.parse::<f64>().unwrap();
+                pieces.push(round_num(f64_val, round_places));
             } else {
                 pieces.push(sum);
             }
@@ -616,13 +639,19 @@ impl Stats {
             pieces.push(empty());
         }
 
-        // min/max
-        if let Some(mm) = self.minmax.as_ref().and_then(|mm| mm.show(typ)) {
+        // min/max/range
+        if let Some(mm) = self
+            .minmax
+            .as_ref()
+            .and_then(|mm| mm.show(typ, round_places))
+        {
             pieces.push(mm.0);
             pieces.push(mm.1);
+            pieces.push(mm.2);
         } else {
             pieces.push(empty());
             pieces.push(empty());
+            pieces.push(empty());
         }
 
         // min/max length
@@ -645,16 +674,20 @@ impl Stats {
                 pieces.push(round_num(v.stddev(), round_places));
                 pieces.push(round_num(v.variance(), round_places));
             } else {
-                pieces.push(unixtime_to_isodate(v.mean()));
+                pieces.push(timestamp_ms_to_rfc3339(v.mean() as i64, typ));
                 // instead of returning stdev in seconds, let's return it in
                 // days as it easier to handle
-                // 86_400 = number of seconds in a day, always round to 5 decimal places
-                pieces.push(round_num(v.stddev() / 86_400_f64, 5));
+                // 86_400_000 = number of milliseconds in a day.
+                // Round to at least 5 decimal places
+                pieces.push(round_num(
+                    v.stddev() / 86_400_000.0_f64,
+                    u32::max(round_places, 5),
+                ));
                 // we don't know how to compute variance on timestamps
                 // it appears the current algorithm we use is not suited to the large timestamp
                 // values as the values we got during testing don't make sense, so
-                // leave it empty for now TODO: use alternate algorithms that are
-                // not sensitive to numerical instability and arithmetic overflow
+                // leave it empty for now
+                // TODO: explore alternate algorithms for calculating variance
                 // see https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
                 pieces.push(empty());
             }
@@ -677,7 +710,7 @@ impl Stats {
             }
         }) {
             if typ == TDateTime || typ == TDate {
-                pieces.push(unixtime_to_isodate(v));
+                pieces.push(timestamp_ms_to_rfc3339(v as i64, typ));
             } else {
                 pieces.push(round_num(v, round_places));
             }
@@ -723,20 +756,23 @@ impl Stats {
 
                 // calculate skewness using Quantile-based measures
                 // https://en.wikipedia.org/wiki/Skewness#Quantile-based_measures
-                // skewness = (q3 - (2.0 * q2) + q1) / iqr
+                // https://blogs.sas.com/content/iml/2017/07/19/quantile-skewness.html
+                // skewness = ((q3 - q2) - (q2 - q1)) / iqr;
+                // which is also skewness = (q3 - (2.0 * q2) + q1) / iqr
+                // which in turn, is the basis of the fused multiply add version below
                 let skewness = (2.0f64.mul_add(-q2, q3) + q1) / iqr;
 
                 if typ == TDateTime || typ == TDate {
-                    pieces.push(unixtime_to_isodate(lof));
-                    pieces.push(unixtime_to_isodate(lif));
+                    pieces.push(timestamp_ms_to_rfc3339(lof as i64, typ));
+                    pieces.push(timestamp_ms_to_rfc3339(lif as i64, typ));
 
-                    pieces.push(unixtime_to_isodate(q1));
-                    pieces.push(unixtime_to_isodate(q2)); // q2 = median
-                    pieces.push(unixtime_to_isodate(q3));
+                    pieces.push(timestamp_ms_to_rfc3339(q1 as i64, typ));
+                    pieces.push(timestamp_ms_to_rfc3339(q2 as i64, typ)); // q2 = median
+                    pieces.push(timestamp_ms_to_rfc3339(q3 as i64, typ));
                     pieces.push(round_num(iqr, round_places));
 
-                    pieces.push(unixtime_to_isodate(uif));
-                    pieces.push(unixtime_to_isodate(uof));
+                    pieces.push(timestamp_ms_to_rfc3339(uif as i64, typ));
+                    pieces.push(timestamp_ms_to_rfc3339(uof as i64, typ));
                 } else {
                     pieces.push(round_num(lof, round_places));
                     pieces.push(round_num(lif, round_places));
@@ -756,26 +792,71 @@ impl Stats {
         // mode/modes & cardinality
         match self.modes.as_mut() {
             None => {
-                if self.which.mode {
+                if self.which.cardinality {
                     pieces.push(empty());
                 }
-                if self.which.cardinality {
+                if self.which.mode {
+                    pieces.push(empty());
+                    pieces.push(empty());
+                    pieces.push(empty());
                     pieces.push(empty());
                 }
             }
             Some(ref mut v) => {
-                if self.which.mode {
-                    pieces.push(
-                        v.modes()
-                            .iter()
-                            .map(|c| unsafe { String::from_utf8_unchecked(c.clone()) })
-                            .join(","),
-                    );
-                }
                 if self.which.cardinality {
                     let mut buffer = itoa::Buffer::new();
                     pieces.push(buffer.format(v.cardinality()).to_owned());
                 }
+                if self.which.mode {
+                    // mode/s
+                    let (modes_result, mode_occurrences) = v.modes();
+                    let modes_count = modes_result.len();
+                    let modes_list = modes_result
+                        .iter()
+                        .map(|c| String::from_utf8_lossy(c))
+                        .join(",");
+                    pieces.push(modes_list);
+                    pieces.push(modes_count.to_string());
+                    pieces.push(mode_occurrences.to_string());
+
+                    // antimode/s
+                    if mode_occurrences == 0 {
+                        // all the values are unique
+                        // so instead of returning everything, just say *ALL
+                        pieces.push("*ALL".to_string());
+                        pieces.push("0".to_string());
+                        pieces.push("1".to_string());
+                    } else {
+                        let (antimodes_result, antimodes_count, antimode_occurrences) =
+                            v.antimodes();
+                        let mut antimodes_list;
+
+                        // We only show the first 10 antimodes
+                        if antimodes_count > 10 {
+                            antimodes_list = "*PREVIEW: ".to_string();
+                            let preview = antimodes_result
+                                .iter()
+                                .map(|c| String::from_utf8_lossy(c))
+                                .take(10)
+                                .join(",");
+                            antimodes_list.push_str(&preview);
+                        } else {
+                            antimodes_list = antimodes_result
+                                .iter()
+                                .map(|c| String::from_utf8_lossy(c))
+                                .join(",");
+                        }
+                        // and truncate at 100 characters and add an ellipsis
+                        if antimodes_list.len() > 100 {
+                            antimodes_list.truncate(100);
+                            antimodes_list.push_str("...");
+                        }
+
+                        pieces.push(antimodes_list);
+                        pieces.push(antimodes_count.to_string());
+                        pieces.push(antimode_occurrences.to_string());
+                    }
+                }
             }
         }
         csv::StringRecord::from(pieces)
@@ -847,13 +928,12 @@ impl FieldType {
             if let Ok(parsed_date) =
                 parse_with_preference(string, DMY_PREFERENCE.load(Ordering::Relaxed))
             {
-                // with rfc3339 format, time component
-                // starts at position 17. If its shorter than 17,
-                // its a plain date, otherwise, its a datetime.
-                if parsed_date.to_string().len() >= 17 {
-                    return TDateTime;
+                // get date in rfc3339 format, if it ends with "T00:00:00+00:00"
+                // its a Date type, otherwise, its DateTime.
+                if parsed_date.to_rfc3339().ends_with("T00:00:00+00:00") {
+                    return TDate;
                 }
-                return TDate;
+                return TDateTime;
             }
         }
         TString
@@ -947,7 +1027,7 @@ impl TypedSum {
                 if let Some(ref mut float) = self.float {
                     *float += from_bytes::<f64>(sample);
                 } else {
-                    // so we don't panic on overflow, use saturating_add
+                    // so we don't panic on overflow/underflow, use saturating_add
                     self.integer = self.integer.saturating_add(from_bytes::<i64>(sample));
                 }
             }
@@ -991,15 +1071,15 @@ impl Commute for TypedSum {
     }
 }
 
-/// `TypedMinMax` keeps track of minimum/maximum values for each possible type
-/// where min/max makes sense.
+/// `TypedMinMax` keeps track of minimum/maximum/range values for each possible type
+/// where min/max/range makes sense.
 #[derive(Clone, Default)]
 struct TypedMinMax {
     strings:  MinMax<Vec<u8>>,
     str_len:  MinMax<usize>,
     integers: MinMax<i64>,
     floats:   MinMax<f64>,
-    dates:    MinMax<String>,
+    dates:    MinMax<i64>,
 }
 
 impl TypedMinMax {
@@ -1010,6 +1090,8 @@ impl TypedMinMax {
             return;
         }
         self.strings.add(sample.to_vec());
+        // we can use unwrap_unchecked with confidence
+        // below since we know the data type domains of the sample
         match typ {
             TString | TNull => {}
             TFloat => {
@@ -1035,14 +1117,14 @@ impl TypedMinMax {
                 self.floats.add(n as f64);
             }
             TDate | TDateTime => {
-                let n = unsafe {
+                let dt = unsafe {
                     parse_with_preference(
-                        str::from_utf8_unchecked(sample),
+                        &String::from_utf8_lossy(sample),
                         DMY_PREFERENCE.load(Ordering::Relaxed),
                     )
                     .unwrap_unchecked()
                 };
-                self.dates.add(n.to_string());
+                self.dates.add(dt.timestamp_millis());
             }
         }
     }
@@ -1059,14 +1141,14 @@ impl TypedMinMax {
         }
     }
 
-    fn show(&self, typ: FieldType) -> Option<(String, String)> {
+    fn show(&self, typ: FieldType, round_places: u32) -> Option<(String, String, String)> {
         match typ {
             TNull => None,
             TString => {
                 if let (Some(min), Some(max)) = (self.strings.min(), self.strings.max()) {
                     let min = String::from_utf8_lossy(min).to_string();
                     let max = String::from_utf8_lossy(max).to_string();
-                    Some((min, max))
+                    Some((min, max, String::new()))
                 } else {
                     None
                 }
@@ -1077,6 +1159,7 @@ impl TypedMinMax {
                     Some((
                         buffer.format(*min).to_owned(),
                         buffer.format(*max).to_owned(),
+                        buffer.format(*max - *min).to_owned(),
                     ))
                 } else {
                     None
@@ -1088,14 +1171,24 @@ impl TypedMinMax {
                     Some((
                         buffer.format(*min).to_owned(),
                         buffer.format(*max).to_owned(),
+                        round_num(*max - *min, round_places),
                     ))
                 } else {
                     None
                 }
             }
-            TDate | TDateTime => {
+            TDateTime | TDate => {
                 if let (Some(min), Some(max)) = (self.dates.min(), self.dates.max()) {
-                    Some((min.to_string(), max.to_string()))
+                    Some((
+                        timestamp_ms_to_rfc3339(*min, typ),
+                        timestamp_ms_to_rfc3339(*max, typ),
+                        // 86_400_000 = number of milliseconds per day
+                        #[allow(clippy::cast_precision_loss)]
+                        round_num(
+                            (*max - *min) as f64 / 86_400_000.0_f64,
+                            u32::max(round_places, 5),
+                        ),
+                    ))
                 } else {
                     None
                 }
