@@ -1,31 +1,4 @@
-use crate::config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY};
-use crate::util;
-use crate::CliError;
-use crate::CliResult;
-use anyhow::{anyhow, Result};
-use csv::ByteRecord;
-#[cfg(any(feature = "full", feature = "lite"))]
-use indicatif::{ProgressBar, ProgressDrawTarget};
-use jsonschema::paths::PathChunk;
-use jsonschema::{output::BasicOutput, JSONSchema};
-#[allow(unused_imports)]
-use log::{debug, info};
-use rayon::prelude::*;
-use serde::{Deserialize, Serialize};
-use serde_json::{json, value::Number, Map, Value};
-use std::{env, fs::File, io::BufReader, io::BufWriter, io::Read, io::Write, str};
-use thousands::Separable;
-
-macro_rules! fail {
-    ($mesg:expr) => {
-        Err(CliError::Other($mesg))
-    };
-}
-
-// number of CSV rows to process in a batch
-const BATCH_SIZE: usize = 24_000;
-
-static USAGE: &str = "
+static USAGE: &str = r#"
 Validate CSV data with JSON Schema, and put invalid records into a separate file.
 When run without JSON Schema, only a simple CSV check (RFC 4180) is performed.
 
@@ -37,8 +10,14 @@ Example output files from `mydata.csv`. If piped from stdin, then filename is `s
 
 JSON Schema can be a local file or a URL.
 
+Returns exitcode 0 when the CSV file is valid, exitcode > 0 otherwise.
+If all records are valid, no output files are produced.
+
+For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_validate.rs.
+
 Usage:
     qsv validate [options] [<input>] [<json-schema>]
+    qsv validate --help
 
 Validate options:
     --fail-fast                Stops on first error.
@@ -50,7 +29,9 @@ Validate options:
     -j, --jobs <arg>           The number of jobs to run in parallel.
                                When not set, the number of jobs is set to the
                                number of CPUs detected.
-
+    -b, --batch <size>         The number of rows per batch to load into memory,
+                               before running in parallel.
+                               [default: 50000]
 
 Common options:
     -h, --help                 Display this message
@@ -60,43 +41,96 @@ Common options:
                                appear as the header row in the output.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. [default: ,]
-    -q, --quiet                Don't show progress bars.
-";
+    -p, --progressbar          Show progress bars. Not valid for stdin.
+"#;
+
+use std::{
+    env,
+    fs::File,
+    io::{BufReader, BufWriter, Read, Write},
+    str,
+};
+
+use csv::ByteRecord;
+#[cfg(any(feature = "full", feature = "lite"))]
+use indicatif::{ProgressBar, ProgressDrawTarget};
+use itertools::Itertools;
+use jsonschema::{output::BasicOutput, paths::PathChunk, JSONSchema};
+use log::info;
+use once_cell::sync::OnceCell;
+use rayon::prelude::*;
+use serde::{Deserialize, Serialize};
+use serde_json::{json, value::Number, Map, Value};
+use thousands::Separable;
+
+use crate::{
+    config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
+    util, CliResult,
+};
+
+// to save on repeated init/allocs
+static NULL_TYPE: once_cell::sync::OnceCell<Value> = OnceCell::new();
 
 #[derive(Deserialize)]
+#[allow(dead_code)]
 struct Args {
-    flag_fail_fast: bool,
-    flag_valid: Option<String>,
-    flag_invalid: Option<String>,
-    flag_json: bool,
+    flag_fail_fast:   bool,
+    flag_valid:       Option<String>,
+    flag_invalid:     Option<String>,
+    flag_json:        bool,
     flag_pretty_json: bool,
-    flag_jobs: Option<usize>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
-    flag_quiet: bool,
-    arg_input: Option<String>,
-    arg_json_schema: Option<String>,
+    flag_jobs:        Option<usize>,
+    flag_batch:       u32,
+    flag_no_headers:  bool,
+    flag_delimiter:   Option<Delimiter>,
+    flag_progressbar: bool,
+    arg_input:        Option<String>,
+    arg_json_schema:  Option<String>,
 }
 
 #[derive(Serialize, Deserialize)]
 struct RFC4180Struct {
     delimiter_char: char,
-    header_row: bool,
-    quote_char: char,
-    num_records: u64,
-    num_fields: usize,
-    fields: Vec<String>,
+    header_row:     bool,
+    quote_char:     char,
+    num_records:    u64,
+    num_fields:     usize,
+    fields:         Vec<String>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
 
+    #[cfg(any(feature = "full", feature = "lite"))]
     let mut rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers);
+    #[cfg(feature = "datapusher_plus")]
+    let rconfig = Config::new(&args.arg_input)
+        .delimiter(args.flag_delimiter)
+        .no_headers(args.flag_no_headers);
 
     let mut rdr = rconfig.reader()?;
 
+    // prep progress bar
+    #[cfg(any(feature = "full", feature = "lite"))]
+    let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
+
+    #[cfg(any(feature = "full", feature = "lite"))]
+    let show_progress =
+        (args.flag_progressbar || std::env::var("QSV_PROGRESSBAR").is_ok()) && !rconfig.is_stdin();
+
+    #[cfg(any(feature = "full", feature = "lite"))]
+    if show_progress {
+        // for full row count, prevent CSV reader from aborting on inconsistent column count
+        rconfig = rconfig.flexible(true);
+        let record_count = util::count_rows(&rconfig)?;
+        rconfig = rconfig.flexible(false);
+        util::prep_progress(&progress, record_count);
+    } else {
+        progress.set_draw_target(ProgressDrawTarget::hidden());
+    }
+
     // if no json schema supplied, only let csv reader RFC4180-validate csv file
     if args.arg_json_schema.is_none() {
         // just read csv file and let csv reader report problems
@@ -128,38 +162,52 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         });
                         return fail!(header_error.to_string());
                     }
-                    return fail!(format!("Cannot read header ({e})."));
+                    return fail_clierror!("Cannot read header ({e}).");
                 }
             }
         }
 
         let mut record_count: u64 = 0;
         for result in rdr.records() {
+            #[cfg(any(feature = "full", feature = "lite"))]
+            if show_progress {
+                progress.inc(1);
+            }
+
             if let Err(e) = result {
                 if args.flag_json || args.flag_pretty_json {
                     let validation_error = json!({
                         "errors": [{
                             "title" : "Validation error",
-                            "detail" : format!("{e}")
+                            "detail" : format!("Last valid row: {record_count} - {e}")
                         }]
                     });
                     return fail!(validation_error.to_string());
                 }
-                return fail!(format!(
-                    r#"Validation error: {e}. Try "qsv fixlengths" or "qsv fmt" to fix it."#
-                ));
+                return fail_clierror!(
+                    r#"Validation error: {e}.\n Last valid row: {record_count}\nTry "qsv fixlengths" or "qsv fmt" to fix it."#
+                );
             }
             record_count += 1;
         }
 
+        #[cfg(any(feature = "full", feature = "lite"))]
+        if show_progress {
+            progress.set_message(format!(
+                " validated {} records.",
+                progress.length().unwrap().separate_with_commas()
+            ));
+            util::finish_progress(&progress);
+        }
+
         let msg = if args.flag_json || args.flag_pretty_json {
             let rfc4180 = RFC4180Struct {
                 delimiter_char: rconfig.get_delimiter() as char,
-                header_row: !rconfig.no_headers,
-                quote_char: rconfig.quote as char,
-                num_records: record_count,
-                num_fields: header_len,
-                fields: field_vec,
+                header_row:     !rconfig.no_headers,
+                quote_char:     rconfig.quote as char,
+                num_records:    record_count,
+                num_fields:     header_len,
+                fields:         field_vec,
             };
 
             if args.flag_pretty_json {
@@ -173,8 +221,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 record_count.separate_with_commas()
             )
         };
-        info!("{msg}");
-        println!("{msg}");
+        woutinfo!("{msg}");
 
         return Ok(());
     }
@@ -182,24 +229,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let headers = rdr.byte_headers()?.clone();
     let headers_len = headers.len();
 
-    // prep progress bar
-    #[cfg(any(feature = "full", feature = "lite"))]
-    let progress = ProgressBar::new(0);
-
-    // for purpose of full file row count, prevent CSV reader to abort on inconsistent column count
-    rconfig = rconfig.flexible(true);
-    let record_count = util::count_rows(&rconfig)?;
-    rconfig = rconfig.flexible(false);
-
-    #[cfg(any(feature = "full", feature = "lite"))]
-    if args.flag_quiet {
-        progress.set_draw_target(ProgressDrawTarget::hidden());
-    } else {
-        util::prep_progress(&progress, record_count);
-    }
-    #[cfg(any(feature = "full", feature = "lite"))]
-    let not_quiet = !args.flag_quiet;
-
     // parse and compile supplied JSON Schema
     let (schema_json, schema_compiled): (Value, JSONSchema) =
         match load_json(&args.arg_json_schema.unwrap()) {
@@ -211,69 +240,72 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         match JSONSchema::options().compile(&json) {
                             Ok(schema) => (json, schema),
                             Err(e) => {
-                                return fail!(format!("Cannot compile schema json. error: {e}"));
+                                return fail_clierror!("Cannot compile schema json. error: {e}");
                             }
                         }
                     }
                     Err(e) => {
-                        return fail!(format!("Unable to parse schema json. error: {e}"));
+                        return fail_clierror!("Unable to parse schema json. error: {e}");
                     }
                 }
             }
             Err(e) => {
-                return fail!(format!("Unable to retrieve json. error: {e}"));
+                return fail_clierror!("Unable to retrieve json. error: {e}");
             }
         };
 
     // debug!("compiled schema: {:?}", &schema_compiled);
 
     // how many rows read and processed as batches
-    let mut row_number: usize = 0;
+    let mut row_number: u32 = 0;
     // how many invalid rows found
-    let mut invalid_count: usize = 0;
+    let mut invalid_count: u32 = 0;
 
     // amortize memory allocation by reusing record
-    #[allow(unused_assignments)]
     let mut record = csv::ByteRecord::new();
     // reuse batch buffer
-    let mut batch = Vec::with_capacity(BATCH_SIZE);
-    let mut validation_results = Vec::with_capacity(BATCH_SIZE);
-    let mut valid_flags: Vec<bool> = Vec::with_capacity(record_count as usize);
+    let batch_size = args.flag_batch as usize;
+    let mut batch = Vec::with_capacity(batch_size);
+    let mut validation_results = Vec::with_capacity(batch_size);
+    let mut valid_flags: Vec<bool> = Vec::with_capacity(batch_size);
     let mut validation_error_messages: Vec<String> = Vec::with_capacity(50);
 
     // set RAYON_NUM_THREADS
     util::njobs(args.flag_jobs);
 
+    // set this once, as this is used repeatedly in a hot loop
+    NULL_TYPE.set(Value::String("null".to_string())).unwrap();
+
     // main loop to read CSV and construct batches for parallel processing.
     // each batch is processed via Rayon parallel iterator.
     // loop exits when batch is empty.
-    loop {
-        for _ in 0..BATCH_SIZE {
+    'batch_loop: loop {
+        for _ in 0..batch_size {
             match rdr.read_byte_record(&mut record) {
                 Ok(has_data) => {
                     if has_data {
                         row_number += 1;
-                        record.push_field(row_number.to_string().as_bytes());
+                        let mut buffer = itoa::Buffer::new();
+                        record.push_field(buffer.format(row_number).as_bytes());
+
                         // non-allocating trimming in place is much faster on the record level
-                        // with our csv fork than doing field-by-field std::str::trim which is allocating
+                        // with our csv fork than doing per field std::str::trim which is allocating
                         record.trim();
-                        batch.push(record.to_owned());
+                        batch.push(record.clone());
                     } else {
                         // nothing else to add to batch
                         break;
                     }
                 }
                 Err(e) => {
-                    return Err(CliError::Other(format!(
-                        "Error reading row: {row_number}: {e}"
-                    )));
+                    return fail_clierror!("Error reading row: {row_number}: {e}");
                 }
             }
         }
 
         if batch.is_empty() {
             // break out of infinite loop when at EOF
-            break;
+            break 'batch_loop;
         }
 
         // do actual validation via Rayon parallel iterator
@@ -292,7 +324,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .collect_into_vec(&mut validation_results);
 
         // write to validation error report, but keep Vec<bool> to gen valid/invalid files later
-        // because Rayon collect() guaranteeds original order, can sequentially append results to vector with each batch
+        // because Rayon collect() guaranteeds original order, can sequentially append results to
+        // vector with each batch
         for result in &validation_results {
             if let Some(validation_error_msg) = result {
                 invalid_count += 1;
@@ -305,32 +338,31 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         }
 
         #[cfg(any(feature = "full", feature = "lite"))]
-        if not_quiet {
+        if show_progress {
             progress.inc(batch.len() as u64);
         }
         batch.clear();
 
         // for fail-fast, exit loop if batch has any error
         if args.flag_fail_fast && invalid_count > 0 {
-            break;
+            break 'batch_loop;
         }
-    } // end infinite loop
+    } // end batch loop
 
     #[cfg(any(feature = "full", feature = "lite"))]
-    if not_quiet {
+    if show_progress {
         progress.set_message(format!(
             " validated {} records.",
-            progress.length().separate_with_commas()
+            progress.length().unwrap().separate_with_commas()
         ));
         util::finish_progress(&progress);
     }
 
     // only write out invalid/valid/errors output files if there are actually invalid records.
-    // if 100% invalid, then valid file is not needed. but this is rare so live with creating empty file.
+    // if 100% invalid, valid file is not needed, but this is rare so OK with creating empty file.
     if invalid_count > 0 {
         let msg = "Writing invalid/valid/error files...";
-        info!("{msg}");
-        println!("{msg}");
+        woutinfo!("{msg}");
 
         let input_path = args
             .arg_input
@@ -350,28 +382,25 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             &valid_suffix,
             &invalid_suffix,
         )?;
-    }
 
-    // done with validation; print output
-    if args.flag_fail_fast {
-        let msg = format!(
-            "fail-fast enabled. stopped after row {}.\n{} out of {} records invalid.",
-            row_number.separate_with_commas(),
+        // done with validation; print output
+        let fail_fast_msg = if args.flag_fail_fast {
+            format!(
+                "fail-fast enabled. stopped after row {}.\n",
+                row_number.separate_with_commas()
+            )
+        } else {
+            String::new()
+        };
+
+        return fail_clierror!(
+            "{fail_fast_msg}{} out of {} records invalid.",
             invalid_count.separate_with_commas(),
             row_number.separate_with_commas()
         );
-        info!("{msg}");
-        println!("{msg}");
-    } else {
-        let msg = format!(
-            "{} out of {} records invalid.",
-            invalid_count.separate_with_commas(),
-            row_number.separate_with_commas()
-        );
-        info!("{msg}");
-        println!("{msg}");
     }
 
+    winfo!("All {} records valid.", row_number.separate_with_commas());
     Ok(())
 }
 
@@ -436,7 +465,8 @@ fn write_error_report(input_path: &str, validation_error_messages: Vec<String>)
     // write out error report
     for error_msg in validation_error_messages {
         output_writer.write_all(error_msg.as_bytes())?;
-        // since writer is buffered, it's more efficient to do additional write than append Newline to message
+        // since writer is buffered, it's more efficient to do additional write than append Newline
+        // to message
         output_writer.write_all(&[b'\n'])?;
     }
 
@@ -447,7 +477,6 @@ fn write_error_report(input_path: &str, validation_error_messages: Vec<String>)
 }
 
 /// if given record is valid, return None, otherwise, error file entry string
-#[inline]
 fn do_json_validation(
     headers: &ByteRecord,
     headers_len: usize,
@@ -470,7 +499,6 @@ fn do_json_validation(
         schema_compiled,
     )
     .map(|validation_errors| {
-        use itertools::Itertools;
         // squash multiple errors into one long String with linebreaks
         let combined_errors: String = validation_errors
             .iter()
@@ -485,30 +513,37 @@ fn do_json_validation(
 }
 
 /// convert CSV Record into JSON instance by referencing Type from Schema
+#[inline]
 fn to_json_instance(
     headers: &ByteRecord,
     headers_len: usize,
     record: &ByteRecord,
     schema: &Value,
-) -> Result<Value> {
+) -> Result<Value, String> {
     // make sure schema has expected structure
-    let schema_properties = schema
-        .get("properties")
-        .expect("JSON Schema missing 'properties' object");
+    let Some(schema_properties) = schema.get("properties") else {
+                     return fail!("JSON Schema missing 'properties' object");
+                 };
 
     // map holds individual CSV fields converted as serde_json::Value
     // we use with_capacity to minimize allocs
     let mut json_object_map: Map<String, Value> = Map::with_capacity(headers_len);
 
-    let null_type: Value = Value::String("null".to_string());
+    let null_type = unsafe { NULL_TYPE.get_unchecked() };
 
     // iterate over each CSV field and convert to JSON type
     for (i, header) in headers.iter().enumerate() {
         // convert csv header to string
         let header_string = unsafe { std::str::from_utf8_unchecked(header).to_string() };
-        // convert csv value to string; no trimming is done as that's done on the record level beforehand
+        // convert csv value to string; no trimming reqd as it's done on the record level beforehand
         let value_string = unsafe { std::str::from_utf8_unchecked(&record[i]).to_string() };
 
+        // if value_string is empty, then just put an empty JSON String
+        if value_string.is_empty() {
+            json_object_map.insert(header_string, Value::Null);
+            continue;
+        }
+
         // get json type from schema; defaults to STRING if not specified
         let field_def: &Value = schema_properties
             .get(&header_string)
@@ -516,75 +551,80 @@ fn to_json_instance(
 
         let field_type_def: &Value = field_def.get("type").unwrap_or(&Value::Null);
 
-        let json_type = match field_type_def {
-            Value::String(s) => s,
+        // we do json_type as a u8 so match will be a tad faster below
+        let json_type: u8 = match field_type_def {
+            Value::String(s) => s.as_bytes()[0],
             Value::Array(vec) => {
                 // if can't find usable type info, defaults to "string"
-                let mut return_val = "string";
+                let mut return_val = b's';
 
                 // grab the first entry that's not a "null", since it just means value is optional
                 for val in vec {
-                    if *val == null_type {
+                    if *val == *null_type {
                         // keep looking
                         continue;
                     }
-                    return_val = val.as_str().expect("type info should be a JSON string");
+                    return_val = if let Some(s) = val.as_str() {
+                        s.as_bytes()[0]
+                    } else {
+                        return fail!("type info should be a JSON string");
+                    };
                 }
 
                 return_val
             }
             _ => {
                 // default to JSON String
-                "string"
+                b's'
             }
         };
 
         // dbg!(i, &header_string, &value_string, &json_type);
 
-        // if value_string is empty, then just put an empty JSON String
-        if value_string.is_empty() {
-            json_object_map.insert(header_string, Value::Null);
-            continue;
-        }
-
+        // matching against a u8, rather than an str should a tad faster
         match json_type {
-            "string" => {
+            b's' => {
+                // string
                 json_object_map.insert(header_string, Value::String(value_string));
             }
-            "number" => {
+            b'n' => {
+                // number
                 if let Ok(float) = value_string.parse::<f64>() {
                     json_object_map.insert(
                         header_string,
                         Value::Number(Number::from_f64(float).expect("not a valid f64 float")),
                     );
                 } else {
-                    return Err(anyhow!(
-                        "Can't cast into Float. header: {header_string}, value: {value_string}, json type: {json_type}"
-                    ));
+                    return fail_format!(
+                        "Can't cast into Float. header: {header_string}, value: {value_string}, \
+                         json type: number"
+                    );
                 }
             }
-            "integer" => {
+            b'i' => {
+                // integer
                 if let Ok(int) = value_string.parse::<i64>() {
                     json_object_map.insert(header_string, Value::Number(Number::from(int)));
                 } else {
-                    return Err(anyhow!(
-                        "Can't cast into Integer. header: {header_string}, value: {value_string}, json type: {json_type}"
-                    ));
+                    return fail_format!(
+                        "Can't cast into Integer. header: {header_string}, value: {value_string}, \
+                         json type: integer"
+                    );
                 }
             }
-            "boolean" => {
+            b'b' => {
+                // boolean
                 if let Ok(boolean) = value_string.parse::<bool>() {
                     json_object_map.insert(header_string, Value::Bool(boolean));
                 } else {
-                    return Err(anyhow!(
-                        "Can't cast into Boolean. header: {header_string}, value: {value_string}, json type: {json_type}"
-                    ));
+                    return fail_format!(
+                        "Can't cast into Boolean. header: {header_string}, value: {value_string}, \
+                         json type: boolean"
+                    );
                 }
             }
             _ => {
-                return Err(anyhow!(
-                    "Unsupported JSON type. header: {header_string}, value: {value_string}, json type: {json_type}"
-                ));
+                unreachable!("we should never get an unknown json type");
             }
         }
     }
@@ -597,9 +637,10 @@ fn to_json_instance(
 #[cfg(test)]
 mod tests_for_csv_to_json_conversion {
 
-    use super::*;
     use serde_json::json;
 
+    use super::*;
+
     /// get schema used for unit tests
     fn schema_json() -> Value {
         // from https://json-schema.org/learn/miscellaneous-examples.html
@@ -652,6 +693,7 @@ mod tests_for_csv_to_json_conversion {
     #[test]
     #[allow(clippy::approx_constant)]
     fn test_to_json_instance() {
+        let _ = NULL_TYPE.get_or_init(|| Value::String("null".to_string()));
         let csv = "A,B,C,D,E,F,G,H,I,J,K,L
         hello,3.1415,300000000,true,,,,,hello,3.1415,300000000,true";
 
@@ -662,11 +704,11 @@ mod tests_for_csv_to_json_conversion {
 
         assert_eq!(
             to_json_instance(&headers, headers.len(), &record, &schema_json())
-                .expect("can convert csv to json instance"),
+                .expect("can't convert csv to json instance"),
             json!({
                 "A": "hello",
                 "B": 3.1415,
-                "C": 300000000,
+                "C": 300_000_000,
                 "D": true,
                 "E": null,
                 "F": null,
@@ -674,7 +716,7 @@ mod tests_for_csv_to_json_conversion {
                 "H": null,
                 "I": "hello",
                 "J": 3.1415,
-                "K": 300000000,
+                "K": 300_000_000,
                 "L": true,
             })
         );
@@ -682,6 +724,7 @@ mod tests_for_csv_to_json_conversion {
 
     #[test]
     fn test_to_json_instance_cast_integer_error() {
+        let _ = NULL_TYPE.get_or_init(|| Value::String("null".to_string()));
         let csv = "A,B,C,D,E,F,G,H
         hello,3.1415,3.0e8,true,,,,";
 
@@ -698,13 +741,14 @@ mod tests_for_csv_to_json_conversion {
         let error = result.err().unwrap();
         assert_eq!(
             "Can't cast into Integer. header: C, value: 3.0e8, json type: integer",
-            error.to_string()
+            error
         );
     }
 }
 
 /// Validate JSON instance against compiled JSON schema
 /// If invalid, returns Some(Vec<(String,String)>) holding the error messages
+#[inline]
 fn validate_json_instance(
     instance: &Value,
     schema_compiled: &JSONSchema,
@@ -712,7 +756,9 @@ fn validate_json_instance(
     let validation_output = schema_compiled.apply(instance);
 
     // If validation output is Invalid, then grab field names and errors
-    if !validation_output.flag() {
+    if validation_output.flag() {
+        None
+    } else {
         // get validation errors as String
         let validation_errors: Vec<(String, String)> = match validation_output.basic() {
             BasicOutput::Invalid(errors) => errors
@@ -729,14 +775,12 @@ fn validate_json_instance(
                 })
                 .collect(),
             BasicOutput::Valid(_annotations) => {
-                // shouln't happen
+                // shouldn't happen
                 unreachable!("Unexpected error.");
             }
         };
 
         Some(validation_errors)
-    } else {
-        None
     }
 }
 
@@ -774,11 +818,12 @@ mod tests_for_schema_validation {
     fn compiled_schema() -> JSONSchema {
         JSONSchema::options()
             .compile(&schema_json())
-            .expect("A valid schema")
+            .expect("Invalid schema")
     }
 
     #[test]
     fn test_validate_with_no_errors() {
+        let _ = NULL_TYPE.get_or_init(|| Value::String("null".to_string()));
         let csv = "title,name,age
         Professor,Xaviers,60";
 
@@ -796,6 +841,7 @@ mod tests_for_schema_validation {
 
     #[test]
     fn test_validate_with_error() {
+        let _ = NULL_TYPE.get_or_init(|| Value::String("null".to_string()));
         let csv = "title,name,age
         Professor,X,60";
 
@@ -820,31 +866,44 @@ mod tests_for_schema_validation {
     }
 }
 
-fn load_json(uri: &str) -> Result<String> {
+fn load_json(uri: &str) -> Result<String, String> {
     let json_string = match uri {
         url if url.starts_with("http") => {
-            // dbg!(&url);
             use reqwest::blocking::Client;
-            let client = Client::builder()
+            let client = match Client::builder()
                 .user_agent(util::DEFAULT_USER_AGENT)
                 .brotli(true)
                 .gzip(true)
+                .deflate(true)
+                .use_rustls_tls()
+                .http2_adaptive_window(true)
                 .build()
-                .unwrap();
+            {
+                Ok(c) => c,
+                Err(e) => {
+                    return fail_format!("Cannot build reqwest client: {e}.");
+                }
+            };
 
-            let response = client.get(url).send()?;
-            response.text()?
+            match client.get(url).send() {
+                Ok(response) => response.text().unwrap_or_default(),
+                Err(e) => return fail_format!("Cannot read JSON at url {url}: {e}."),
+            }
         }
         path => {
-            // dbg!(&_path);
             let mut buffer = String::new();
-            BufReader::new(File::open(path)?).read_to_string(&mut buffer)?;
+            match File::open(path) {
+                Ok(p) => {
+                    BufReader::new(p)
+                        .read_to_string(&mut buffer)
+                        .unwrap_or_default();
+                }
+                Err(e) => return fail_format!("Cannot read JSON file {path}: {e}."),
+            }
             buffer
         }
     };
 
-    // dbg!(&json_string);
-
     Ok(json_string)
 }
 
