@@ -378,16 +378,21 @@ impl Config {
     }
 
     fn autoindex_file(&self) {
-        // autoindex should never panic. It should silently fail as its a "convenience fn"
+        use io::prelude::*;
+
+        // autoindex_file should never panic. It should silently fail as its a "convenience fn"
         // that's why we have a lot of let-else returns, in lieu of unwraps
         let Some(path_buf) = &self.path else { return };
-        let Ok(mut rdr) = self.reader_file() else { return };
 
         let pidx = util::idx_path(Path::new(path_buf));
         let Ok(idxfile) = fs::File::create(pidx) else { return };
+        let Ok(mut rdr) = self.reader_file() else { return };
         let mut wtr = io::BufWriter::new(idxfile);
         match csv_index::RandomAccessSimple::create(&mut rdr, &mut wtr) {
-            Ok(_) => debug!("autoindex of {path_buf:?} successful."),
+            Ok(_) => {
+                let Ok(_) = wtr.flush() else { return };
+                debug!("autoindex of {path_buf:?} successful.");
+            }
             Err(e) => debug!("autoindex of {path_buf:?} failed: {e}"),
         }
     }
@@ -407,11 +412,13 @@ impl Config {
                 let idx_file = match fs::File::open(util::idx_path(p)) {
                     Err(e) => {
                         if self.autoindex {
+                            // however, if QSV_AUTOINDEX is set, we create the index automatically
                             self.autoindex_file();
+                            fs::File::open(util::idx_path(p)).unwrap()
                         } else {
                             warn!("No index file found - {p:?}: {e}");
+                            return Ok(None);
                         }
-                        return Ok(None);
                     }
                     Ok(f) => f,
                 };
@@ -420,8 +427,9 @@ impl Config {
             (Some(p), Some(ip)) => (fs::File::open(p)?, fs::File::open(ip)?),
         };
         // If the CSV data was last modified after the index file was last
-        // modified, then return an error and demand the user regenerate the
-        // index.
+        // modified, then return an error and demand the user regenerate the index.
+        // Unless QSV_AUTOINDEX is set, in which case, we'll recreate the
+        // stale index automatically
         let data_modified = util::last_modified(&csv_file.metadata()?);
         let idx_modified = util::last_modified(&idx_file.metadata()?);
         if data_modified > idx_modified {
