@@ -1,5 +1,4 @@
-use std::borrow::ToOwned;
-use std::process;
+use std::{borrow::ToOwned, process};
 
 use crate::workdir::Workdir;
 
@@ -97,7 +96,7 @@ fn test_slice(
                 cmd.arg("--len").arg(&end.to_string());
             } else {
                 cmd.arg("--len")
-                    .arg(&(end - start.abs() as usize).to_string());
+                    .arg(&(end - start.unsigned_abs()).to_string());
             }
         } else {
             cmd.arg("--end").arg(&end.to_string());
