@@ -1,4 +1,3 @@
-#[macro_use]
 extern crate log;
 extern crate serde;
 
@@ -8,10 +7,7 @@ extern crate quickcheck;
 extern crate rand;
 extern crate stats;
 
-use std::env;
-use std::fmt;
-use std::mem::transmute;
-use std::ops;
+use std::{env, fmt, mem::transmute, ops};
 
 use quickcheck::{Arbitrary, Gen, QuickCheck, Testable};
 use rand::{thread_rng, Rng};
@@ -36,40 +32,58 @@ mod workdir;
 
 #[cfg(feature = "apply")]
 mod test_apply;
+#[cfg(feature = "datapusher_plus")]
+mod test_applydp;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_behead;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_cat;
 mod test_combos;
 mod test_comments;
 mod test_count;
 mod test_dedup;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_enumerate;
 mod test_excel;
 mod test_exclude;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_explode;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_extsort;
 #[cfg(feature = "fetch")]
 mod test_fetch;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_fill;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_fixlengths;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_flatten;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_fmt;
 #[cfg(all(feature = "foreach", target_family = "unix"))]
 mod test_foreach;
 mod test_frequency;
 mod test_headers;
 mod test_index;
+mod test_input;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_join;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_jsonl;
-#[cfg(feature = "lua")]
-mod test_lua;
+#[cfg(feature = "luau")]
+mod test_luau;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_partition;
 mod test_pseudo;
 #[cfg(feature = "python")]
 mod test_py;
 mod test_rename;
 mod test_replace;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_reverse;
+mod test_safenames;
 mod test_sample;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_schema;
 mod test_search;
 mod test_searchset;
@@ -77,9 +91,14 @@ mod test_select;
 mod test_slice;
 mod test_sniff;
 mod test_sort;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_split;
 mod test_stats;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_table;
+#[cfg(any(feature = "full", feature = "lite"))]
+mod test_tojsonl;
+#[cfg(any(feature = "full", feature = "lite"))]
 mod test_transpose;
 mod test_validate;
 
@@ -125,20 +144,20 @@ impl CsvRecord {
 impl ops::Deref for CsvRecord {
     type Target = [String];
     fn deref<'a>(&'a self) -> &'a [String] {
-        &*self.0
+        &self.0
     }
 }
 
 #[allow(clippy::needless_lifetimes)]
 impl ops::DerefMut for CsvRecord {
     fn deref_mut<'a>(&'a mut self) -> &'a mut [String] {
-        &mut *self.0
+        &mut self.0
     }
 }
 
 impl fmt::Debug for CsvRecord {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let bytes: Vec<_> = self.iter().map(|s| s.as_bytes()).collect();
+        let bytes: Vec<_> = self.iter().map(std::string::String::as_bytes).collect();
         write!(f, "{:?}", bytes)
     }
 }
@@ -180,7 +199,7 @@ impl CsvData {
     }
 
     fn len(&self) -> usize {
-        (&**self).len()
+        (**self).len()
     }
 
     fn is_empty(&self) -> bool {
@@ -192,7 +211,7 @@ impl CsvData {
 impl ops::Deref for CsvData {
     type Target = [CsvRecord];
     fn deref<'a>(&'a self) -> &'a [CsvRecord] {
-        &*self.data
+        &self.data
     }
 }
 
