@@ -1,14 +1,13 @@
-use std::env;
-use std::fmt;
-use std::fs;
-use std::fs::File;
-use std::io::Write;
-use std::io::{self, Read};
-use std::path::{Path, PathBuf};
-use std::process;
-use std::str::FromStr;
-use std::sync::atomic;
-use std::time::Duration;
+use std::{
+    env, fmt, fs,
+    fs::File,
+    io::{self, Read, Write},
+    path::{Path, PathBuf},
+    process,
+    str::FromStr,
+    sync::atomic,
+    time::Duration,
+};
 
 use crate::Csv;
 
@@ -17,8 +16,8 @@ static QSV_INTEGRATION_TEST_DIR: &str = "xit";
 static NEXT_ID: atomic::AtomicUsize = atomic::AtomicUsize::new(0);
 
 pub struct Workdir {
-    root: PathBuf,
-    dir: PathBuf,
+    root:     PathBuf,
+    dir:      PathBuf,
     flexible: bool,
 }
 
@@ -36,7 +35,7 @@ impl Workdir {
         let dir = root
             .join(QSV_INTEGRATION_TEST_DIR)
             .join(name)
-            .join(&format!("test-{}", id));
+            .join(format!("test-{}", id));
         if let Err(err) = create_dir_all(&dir) {
             panic!("Could not create '{:?}': {}", dir, err);
         }
@@ -60,9 +59,9 @@ impl Workdir {
         let mut wtr = csv::WriterBuilder::new()
             .flexible(self.flexible)
             .delimiter(delim)
-            .from_path(&self.path(name))
+            .from_path(self.path(name))
             .unwrap();
-        for row in rows.to_vecs().into_iter() {
+        for row in rows.to_vecs() {
             wtr.write_record(row).unwrap();
         }
         wtr.flush().unwrap();
@@ -87,6 +86,7 @@ impl Workdir {
         let stdout: String = self.stdout(cmd);
         let mut rdr = csv::ReaderBuilder::new()
             .has_headers(false)
+            .flexible(true)
             .from_reader(io::Cursor::new(stdout));
 
         let records: Vec<Vec<String>> = rdr
@@ -94,13 +94,13 @@ impl Workdir {
             .collect::<Result<Vec<csv::StringRecord>, _>>()
             .unwrap()
             .into_iter()
-            .map(|r| r.iter().map(|f| f.to_string()).collect())
+            .map(|r| r.iter().map(std::string::ToString::to_string).collect())
             .collect();
         Csv::from_vecs(records)
     }
 
     pub fn command(&self, sub_command: &str) -> process::Command {
-        let mut cmd = process::Command::new(&self.qsv_bin());
+        let mut cmd = process::Command::new(self.qsv_bin());
         if sub_command.is_empty() {
             cmd.current_dir(&self.dir);
         } else {
@@ -110,24 +110,7 @@ impl Workdir {
     }
 
     pub fn output(&self, cmd: &mut process::Command) -> process::Output {
-        debug!("[{}]: {:?}", self.dir.display(), cmd);
-        let o = cmd.output().unwrap();
-        if !o.status.success() {
-            panic!(
-                "\n\n===== {:?} =====\n\
-                    command failed but expected success!\
-                    \n\ncwd: {}\
-                    \n\nstatus: {}\
-                    \n\nstdout: {}\n\nstderr: {}\
-                    \n\n=====\n",
-                cmd,
-                self.dir.display(),
-                o.status,
-                String::from_utf8_lossy(&o.stdout),
-                String::from_utf8_lossy(&o.stderr)
-            )
-        }
-        o
+        cmd.output().unwrap()
     }
 
     pub fn run(&self, cmd: &mut process::Command) {
@@ -145,9 +128,8 @@ impl Workdir {
     }
 
     pub fn output_stderr(&self, cmd: &mut process::Command) -> String {
-        debug!("[{}]: {:?}", self.dir.display(), cmd);
-        // ensures stderr has been flushed before we run our cmd
         {
+            // ensures stderr has been flushed before we run our cmd
             let mut _stderr = io::stderr();
             _stderr.flush().unwrap();
         }
@@ -160,25 +142,35 @@ impl Workdir {
         }
     }
 
-    pub fn assert_err(&self, cmd: &mut process::Command) {
+    pub fn assert_success(&self, cmd: &mut process::Command) {
         let o = cmd.output().unwrap();
-        if o.status.success() {
-            panic!(
-                "\n\n===== {:?} =====\n\
-                    command succeeded but expected failure!\
-                    \n\ncwd: {}\
-                    \n\nstatus: {}\
-                    \n\nstdout: {}\n\nstderr: {}\
-                    \n\n=====\n",
-                cmd,
-                self.dir.display(),
-                o.status,
-                String::from_utf8_lossy(&o.stdout),
-                String::from_utf8_lossy(&o.stderr)
-            );
-        }
+        assert!(
+            o.status.success(),
+            "\n\n===== {:?} =====\ncommand failed but expected success!\n\ncwd: {}\n\nstatus: \
+             {}\n\nstdout: {}\n\nstderr: {}\n\n=====\n",
+            cmd,
+            self.dir.display(),
+            o.status,
+            String::from_utf8_lossy(&o.stdout),
+            String::from_utf8_lossy(&o.stderr)
+        );
     }
 
+    pub fn assert_err(&self, cmd: &mut process::Command) {
+        let o = cmd.output().unwrap();
+        assert!(
+            !o.status.success(),
+            "\n\n===== {:?} =====\ncommand succeeded but expected failure!\n\ncwd: {}\n\nstatus: \
+             {}\n\nstdout: {}\n\nstderr: {}\n\n=====\n",
+            cmd,
+            self.dir.display(),
+            o.status,
+            String::from_utf8_lossy(&o.stdout),
+            String::from_utf8_lossy(&o.stderr)
+        );
+    }
+
+    // returns contents of specified file in resources/test directory
     pub fn load_test_resource(&self, filename: &str) -> String {
         // locate resources/test relative to crate base dir
         let mut path = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
@@ -188,6 +180,7 @@ impl Workdir {
         self.from_str::<String>(path.as_path())
     }
 
+    // returns absolute file path in resources/test directory
     pub fn load_test_file(&self, filename: &str) -> String {
         // locate resources/test relative to crate base dir
         let mut path = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
@@ -211,11 +204,23 @@ impl Workdir {
         self.dir.join(name)
     }
 
+    #[cfg(feature = "full")]
     pub fn qsv_bin(&self) -> PathBuf {
         self.root.join("qsv")
     }
 
+    #[cfg(feature = "lite")]
+    pub fn qsv_bin(&self) -> PathBuf {
+        self.root.join("qsvlite")
+    }
+
+    #[cfg(feature = "datapusher_plus")]
+    pub fn qsv_bin(&self) -> PathBuf {
+        self.root.join("qsvdp")
+    }
+
     // clear all files in directory
+    #[cfg(any(feature = "full", feature = "lite"))]
     pub fn clear_contents(&self) -> io::Result<()> {
         for entry in fs::read_dir(&self.dir)? {
             fs::remove_file(entry?.path())?;
@@ -230,7 +235,7 @@ impl fmt::Debug for Workdir {
     }
 }
 
-// For whatever reason, `fs::create_dir_all` fails intermittently on Travis
+// For whatever reason, `fs::create_dir_all` fails intermittently on CI
 // with a weird "file exists" error. Despite my best efforts to get to the
 // bottom of it, I've decided a try-wait-and-retry hack is good enough.
 fn create_dir_all<P: AsRef<Path>>(p: P) -> io::Result<()> {
